
class TicktickListParameters:

    ID = "id"
    NAME = "name"
    IS_OWNER = "isOwner"
    COLOR = "color"
    IN_ALL = "inAll"
    SORT_ORDER = "sortOrder"
    SORT_OPTION = "sortOption"
    SORT_TYPE = "sortType"
    USER_COUNT = "userCount"
    ETAG = "etag"
    MODIFIED_TIME = "modifiedTime"
    CLOSED = "closed"
    MUTED = "muted"
    TRANSFERRED = "transferred"
    GROUP_ID = "groupId"
    VIEW_MODE = "viewMode"
    NOTIFICATION_OPTIONS = "notificationOptions"
    TEAM_ID = "teamId"
    PERMISSION = "permission"
    KIND = "kind"
    TIMELINE = "timeline"
